#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "effects.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "aura.h"
#include "classes.h"
#include "gamerules.h"
#include "chargedattack.h"
#include "magicattack.h"
#include "explodingattack.h"
#include "player.h"
#include "beamattack.h"


#define BIGBANGATTACK_SIZE		70
#define BIGBANG_TRAILLENGTH		80
#define BIGBANG_MAXCHARGETIME	30
#define BIGBANG_KI_COST			25
#define BIGBANG_SPEED			1900
#define BIGBANG_PLAYERSLOWDOWN	0.0
#define BIGBANG_DAMAGE			4
#define BIGBANG_RADIUS			6

extern int gmsgFlare;
/**
 * Big bang attack
 * @version 30-3-2002
 * @author Herwin 'harSens' van Welbergen
 */
class CBigBangAttack:public CBeamAttack
{	
public:
	/**
	* Create a bigbang attack 
	* @param entvars_t *pevOwner: Monster/Player who shoots it
	* @param Vector vecStart: Origin vector of the attack
	* @param Vector vecVelocity: Velocity of the attack
	* @param damage: the amount of damage to be done
	* @param size: the size of the bigbang sprite
	*/
	static CBigBangAttack *ShootBigBang( entvars_t *pevOwner, Vector vecStart, Vector vecVelocity, int damage, float size,short flarea, short flareb, short trail);
};

CBigBangAttack* CBigBangAttack::ShootBigBang( entvars_t *pevOwner, Vector vecStart, Vector vecVelocity, int damage, float size, short flarea, short flareb, short trail)
{	CBigBangAttack *pMagicAttack = GetClassPtr( (CBigBangAttack *)NULL );
	pMagicAttack->Spawn();	
	
	pMagicAttack->pev->movetype = MOVETYPE_FLY;
	pMagicAttack->pev->solid = SOLID_TRIGGER;	//touches everything

	//set model
	SET_MODEL(ENT(pMagicAttack->pev), "sprites/null.spr");	

	pMagicAttack->m_iExplosionColor = BLUE_EXPLOSION;
	
	pMagicAttack->m_iRadius = BIGBANG_RADIUS;

	pMagicAttack->SetDamage(damage);
	pMagicAttack->pev->velocity=vecVelocity;
	pMagicAttack->m_flSpeed=vecVelocity.Length();
	pMagicAttack->pev->angles=UTIL_VecToAngles(pMagicAttack->pev->velocity);
	pMagicAttack->pev->owner=ENT(pevOwner);
	pMagicAttack->pev->classname = MAKE_STRING( "bigbang" );
	
	float flSize = BIGBANGATTACK_SIZE * size;
	float flAttackSize = flSize * 0.25;	//compensate for emptyness in sprites
	UTIL_SetOrigin(pMagicAttack->pev,vecStart + flAttackSize * vecVelocity.Normalize());
	UTIL_SetSize(pMagicAttack->pev, Vector(-flAttackSize, -flAttackSize, -flAttackSize), Vector(flAttackSize, flAttackSize, flAttackSize));
	pMagicAttack->SetTouch( &CBigBangAttack::ExplodeTouch );

	MESSAGE_BEGIN( MSG_ALL, gmsgFlare, NULL );
		WRITE_LONG(pMagicAttack->entindex());		//end entity
		WRITE_SHORT(flarea);						//start attachment
		WRITE_SHORT(flareb);						//start sprites
		WRITE_SHORT(flSize * 0.1);					//size in .1's
		WRITE_SHORT(trail);							//trail sprite
		WRITE_SHORT(BIGBANG_TRAILLENGTH * size);	//trail length
		WRITE_COORD(vecStart.x);					//start pos
		WRITE_COORD(vecStart.y);
		WRITE_COORD(vecStart.z);
	MESSAGE_END();

	pMagicAttack->SetThink( &CBigBangAttack::NormalThink );
	pMagicAttack->pev->nextthink = gpGlobals->time;

	/*DEBUG by harSens: enable bbox	
	extern int gmsgBoundingBox;
	MESSAGE_BEGIN( MSG_ALL, gmsgBoundingBox);
		WRITE_LONG(pMagicAttack->entindex());
	MESSAGE_END();	
	*/
	
	return pMagicAttack;
}
//--------------------------------------------------------------
/**
* Big Bang Weapon
* @version 15-8-2001
* @author Herwin 'harSens' van Welbergen
*/
class CBigBang:public CChargedAttack
{	
public:
	/**
	* Init function
	*/
	void Spawn( void );

	/**
	* Precaches sprites and sounds
	*/
	void Precache( void );

	/**
	* Get  bigbang slot
	* return int: the slot, is 5 for bigbang
	*/
	int iItemSlot( void ) { return 5; }

	/**
	* Get bigbang item info
	* @param ItemInfo *p: The ItemInfo gets stored here
	* @return int: Always returns 1 (why???)
	*/
	int GetItemInfo(ItemInfo *p);
private:
	/**
	 * shoots the bigbang
	 */
	void ShootAttack();

	/**
	 * emits the shooting sound
	 */
	void EmitShootSound();

	/**
	 * emits the charging sound
	 */	
	void EmitChargeSound();
private:
	short m_sFlareA,m_sFlareB,m_sTrail;
};

LINK_ENTITY_TO_CLASS( weapon_bigbang, CBigBang );

void CBigBang::Spawn( void )
{	Precache();
	m_iId = WEAPON_BIGBANG;
	m_iDefaultAmmo = 0;
	m_iClip = -1;

	m_iMaxCharge = BIGBANG_MAXCHARGETIME;
	m_iKiCost = BIGBANG_KI_COST;
	m_flPlayerSlowDown = BIGBANG_PLAYERSLOWDOWN;
	strcpy(m_pAttackName,"bigbang");
	FallInit();// get ready to fall down.
}

void CBigBang::Precache( void )
{	m_sFlareA = PRECACHE_MODEL("sprites/bigbanga.spr");
	m_sFlareB = PRECACHE_MODEL("sprites/bigbangb.spr");
	m_sTrail = PRECACHE_MODEL("sprites/bigbangtrail.spr");
	PRECACHE_SOUND("vegeta/bigbang.wav");
	PRECACHE_SOUND("vegeta/bigbang_charge.wav");
	PRECACHE_SOUND("weapons/bigbang_fire.wav");
}

int CBigBang::GetItemInfo(ItemInfo *p)
{	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "ki";
	p->iMaxAmmo1 = -1;	//max ammo is weapon independent
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = WEAPON_NOCLIP;
	p->iSlot = 4;
	p->iPosition = 4;
	p->iId = m_iId = WEAPON_BIGBANG;
	p->iWeight = BIGBANG_WEIGHT;
	p->iFlags = 0;
	return 1;
}

void CBigBang::EmitShootSound()
{	//make some sounds :-)
	m_pPlayer->EmitClassSound("bigbang.wav");
	EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/bigbang_fire.wav", 1.0, ATTN_NORM);
}

void CBigBang::EmitChargeSound()
{	m_pPlayer->EmitClassSound("bigbang_charge.wav");
}

void CBigBang::ShootAttack()	
{	//shoot the attack
	UTIL_MakeVectors( m_pPlayer->pev->v_angle );
	Vector vecSrc = m_pPlayer->GetGunPosition() + gpGlobals->v_right * 10 + gpGlobals->v_forward * 10;
	Vector vecThrow = gpGlobals->v_forward * BIGBANG_SPEED;
	CBigBangAttack::ShootBigBang( m_pPlayer->pev, vecSrc, vecThrow, m_iCharge * BIGBANG_DAMAGE * GetPowerRatio() , (float)m_iCharge/(float)BIGBANG_MAXCHARGETIME,m_sFlareA,m_sFlareB,m_sTrail);
}