#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "effects.h"
#include "aura.h"
#include "classes.h"
#include "magicattack.h"
#include "explodingattack.h"
#include "player.h"
#include "gamerules.h"
#include "magicattack.h"
#include "explodingattack.h"
#include "beamattack.h"
#include "guidedattack.h"
#include "controlledbeam.h"

/**
* Kamehameha Weapon
* @version redo: 13-7-2001
* @author Herwin 'harSens' van Welbergen
*/
class CKamehameha:public CControlledBeam
{	
public:
	/**
	* Init function
	*/
	void Spawn( void );

	/**
	* Precaches sprites and sounds
	*/
	void Precache( void );

	/**
	* Get kamehameha slot
	* return int: the slot, is 4 for kamehameha
	*/
	int iItemSlot( void ) { return 4; }

	/**
	* Get kamehameha item info
	* @param ItemInfo *p: The ItemInfo gets stored here
	* @return int: Always returns 1 (why???)
	*/
	int GetItemInfo(ItemInfo *p);

private:
	CSprite *m_pChargeSprite;		//the sprites wich appears after charging
	
	/**
	* Emits charge sound
	*/
	virtual void EmitChargeSound();

	/**
	* Emits shoot sound
	*/
	virtual void EmitShootSound();

	/**
	* Inits charge sprite
	*/
	virtual void InitChargeSprite();

	/**
	* Enable charge sprite
	*/
	virtual void EnableChargeSprite(){m_pChargeSprite->SetScale(0.015);m_pChargeSprite->TurnOn();};

	/**
	* Disable charge sprite
	*/
	virtual void DisableChargeSprite(){m_pChargeSprite->TurnOff();};

	/**
	* Grow charge sprite
	*/
	virtual void GrowChargeSprite(){m_pChargeSprite->SetScale((float)m_iCharge * 0.015);};

	/**
	* Remove charge sprite
	*/
	virtual void RemoveChargeSprite(){UTIL_Remove(m_pChargeSprite);};
};

LINK_ENTITY_TO_CLASS( weapon_kamehameha, CKamehameha );

void CKamehameha::Spawn( void )
{	Precache();
	m_eAttackState = ATTACK_IDLE; //not firing
	m_iCharge = 0;
	m_pMagic = NULL;
	m_iId = WEAPON_KAMEHAMEHA;
	m_iDefaultAmmo = 0;
	m_iClip = -1;

	m_iMaxCharge = KAMEHAMEHA_MAXCHARGETIME;
	m_iSpeed = KAMEHAMEHA_SPEED;
	m_iKiCost = KAMEHAMEHA_KI_COST;
	m_iDamage = KAMEHAMEHA_DAMAGE;
	m_iRadius = KAMEHAMEHA_RADIUS;
	m_flPlayerSlowdown = KAMEHAMEHA_PLAYERSLOWDOWN;
	strcpy(m_pAttackName,"kamehameha");
	m_flRateOfTurn = KAMEHAMEHA_ROT;

	//explosion color
	m_iExplosionColor = BLUE_EXPLOSION;
	m_iAttachment =	1;

	FallInit();// get ready to fall down.
}

void CKamehameha::Precache( void )
{	m_sFlareA = PRECACHE_MODEL("sprites/kamehamehaa.spr");
	m_sFlareB = PRECACHE_MODEL("sprites/kamehamehab.spr");
	m_sStartA = PRECACHE_MODEL("sprites/kamehamehaastart.spr");
	m_sStartB = PRECACHE_MODEL("sprites/kamehamehabstart.spr");
	
	PRECACHE_MODEL("sprites/kamecharge.spr");
	PRECACHE_SOUND("goku/kamehame.wav");
	PRECACHE_SOUND("goku/ha.wav");
	PRECACHE_SOUND("krillin/kamehame.wav");
	PRECACHE_SOUND("krillin/ha.wav");
	PRECACHE_SOUND("gohan/kamehame.wav");
	PRECACHE_SOUND("gohan/ha.wav");
	PRECACHE_SOUND("cell/kamehame.wav");
	PRECACHE_SOUND("cell/ha.wav");
	PRECACHE_SOUND("cell2/kamehame.wav");
	PRECACHE_SOUND("cell2/ha.wav");	
	PRECACHE_SOUND("weapons/kame_charge.wav");
	PRECACHE_SOUND("weapons/kamehameha_fire.wav");
	m_sTrail = PRECACHE_MODEL("sprites/kamehamehatrail.spr");
	
	CControlledBeam::Precache();
}

int CKamehameha::GetItemInfo(ItemInfo *p)
{	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "ki";
	p->iMaxAmmo1 = -1;	//max ammo is weapon independent
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = WEAPON_NOCLIP;
	p->iSlot = 3;
	p->iPosition = 1;
	p->iId = m_iId = WEAPON_KAMEHAMEHA;
	p->iWeight = KAMEHAMEHA_WEIGHT;
	p->iFlags = 0;
	return 1;
}

void CKamehameha::InitChargeSprite( void )
{	m_pChargeSprite = CSprite::SpriteCreate( "sprites/kamecharge.spr", m_pPlayer->pev->origin, true);
	m_pChargeSprite->SetAttachment(m_pPlayer->edict(),1);
	m_pChargeSprite->SetTransparency( kRenderTransAdd, 255, 255, 255, 254, kRenderFxGlowShell );
	m_pChargeSprite->SetScale(0.1);
	m_pChargeSprite->pev->framerate = 10;
	m_pChargeSprite->TurnOff();	
}

void CKamehameha::EmitShootSound()
{	m_pPlayer->EmitClassSound("ha.wav");
	EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/kamehameha_fire.wav", 1.0, ATTN_NORM);
}

void CKamehameha::EmitChargeSound()
{	m_pPlayer->EmitClassSound("kamehame.wav");
	EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/kame_charge.wav", 1.0, ATTN_NORM);
}