#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "effects.h"
#include "aura.h"
#include "classes.h"
#include "magicattack.h"
#include "explodingattack.h"
#include "player.h"
#include "gamerules.h"
#include "magicattack.h"
#include "explodingattack.h"
#include "beamattack.h"
#include "guidedattack.h"
#include "controlledbeam.h"

/**
* Powerbeam weapon (modified masenko)
* @version 28-6-2001
* @author Herwin 'harSens' van Welbergen
*/
class CPowerBeam:public CControlledBeam
{	
public:
	/**
	* Init function
	*/
	void Spawn( void );

	/**
	* Precaches sprites and sounds
	*/
	void Precache( void );

	/**
	* Get masenko slot
	* return int: the slot, is 6 for masenko
	*/
	int iItemSlot( void ) { return 6; }

	/**
	* Get masenko item info
	* @param ItemInfo *p: The ItemInfo gets stored here
	* @return int: Always returns 1 (why???)
	*/
	int GetItemInfo(ItemInfo *p);

private:
	CSprite *m_pChargeSprite;		//the sprites wich appears after charging

	/**
	* Emits charge sound
	*/
	virtual void EmitChargeSound();

	/**
	* Emits shoot sound
	*/
	virtual void EmitShootSound();

	/**
	* Inits charge sprite
	*/
	virtual void InitChargeSprite();

	/**
	* Grow charge sprite
	*/
	virtual void GrowChargeSprite(){m_pChargeSprite->SetScale((float)m_iCharge * 0.015);};

	/**
	* Enable charge sprite
	*/
	virtual void EnableChargeSprite(){m_pChargeSprite->SetScale(0.015);m_pChargeSprite->TurnOn();};

	/**
	* Disable charge sprite
	*/
	virtual void DisableChargeSprite(){m_pChargeSprite->TurnOff();};

	/**
	* Remove charge sprite
	*/
	virtual void RemoveChargeSprite(){UTIL_Remove(m_pChargeSprite);};
};

LINK_ENTITY_TO_CLASS( weapon_powerbeam, CPowerBeam);

void CPowerBeam::Spawn( void )
{	Precache();
	m_eAttackState = ATTACK_IDLE; //not firing
	m_iCharge = 0;
	m_pMagic = NULL;
	m_iId = WEAPON_POWERBEAM;
	m_iDefaultAmmo = 0;
	m_iClip = -1;

	m_iMaxCharge = POWERBEAM_MAXCHARGETIME;
	m_iSpeed = POWERBEAM_SPEED;
	m_iKiCost = POWERBEAM_KI_COST;
	m_iDamage = POWERBEAM_DAMAGE;
	m_iRadius = POWERBEAM_RADIUS;
	strcpy(m_pAttackName,"powerbeam");

	//explosion color
	m_iExplosionColor = YELLOW_EXPLOSION;
	m_iAttachment =	1;

	m_flRateOfTurn = POWERBEAM_ROT;

	FallInit();// get ready to fall down.
}

void CPowerBeam::Precache( void )
{	m_sFlareA = PRECACHE_MODEL("sprites/masenkoa.spr");
	m_sFlareB = PRECACHE_MODEL("sprites/masenkob.spr");
	m_sStartA = PRECACHE_MODEL("sprites/masenkoastart.spr");
	m_sStartB = PRECACHE_MODEL("sprites/masenkobstart.spr");

	PRECACHE_MODEL("sprites/masenkocharge.spr");
	PRECACHE_SOUND("gohan/powerbeam.wav");
	PRECACHE_SOUND("gohan/powerbeamha.wav");
	PRECACHE_SOUND("weapons/mase_charge.wav");
	m_sTrail = PRECACHE_MODEL("sprites/masenkotrail.spr");

	CControlledBeam::Precache();
}

int CPowerBeam::GetItemInfo(ItemInfo *p)
{	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "ki";
	p->iMaxAmmo1 = -1;	//max ammo is weapon independent
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = WEAPON_NOCLIP;
	p->iSlot = 5;
	p->iPosition = 4;
	p->iId = m_iId = WEAPON_POWERBEAM;
	p->iWeight = POWERBEAM_WEIGHT;
	p->iFlags = 0;
	return 1;
}

void CPowerBeam::InitChargeSprite( void )
{	m_pChargeSprite = CSprite::SpriteCreate( "sprites/masenkocharge.spr", m_pPlayer->pev->origin, true);
	m_pChargeSprite->SetAttachment(m_pPlayer->edict(),1);
	m_pChargeSprite->SetTransparency( kRenderTransAdd, 255, 255, 255, 254, kRenderFxGlowShell );
	m_pChargeSprite->SetScale(0.1);
	m_pChargeSprite->pev->framerate = 10;
	m_pChargeSprite->TurnOff();	
}

void CPowerBeam::EmitShootSound()
{	m_pPlayer->EmitClassSound("powerbeamha.wav");	
}

void CPowerBeam::EmitChargeSound()
{	m_pPlayer->EmitClassSound("powerbeam.wav");
	EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_WEAPON, "weapons/mase_charge.wav", 1.0, ATTN_NORM);
}